#!/usr/local/bin/perl

package IPAnalyze::Customers;
use strict;
use Sys::SigAction qw(set_sig_handler);

sub new {
	my $me = shift;
	return bless({}, $me);
}

sub cust_ids {
	my $me = shift;
	my $dbh = shift;
	my $key = shift;
	my $value = shift || die "cust_ids syntax error2: value";

	my %criteria = (name => 'Last_name', circuit_id => 'Telephone_number');
	die "invalid key: $key\n" unless($criteria{$key});

	eval {
		my $h = set_sig_handler( 'ALRM', sub { die "timeout!"; } );
		alarm(30);
		my $sql = "SELECT DISTINCT(UidCustomerId) FROM profiles WITH (NOLOCK) WHERE $criteria{$key} ";
		if($key eq 'name'){
			$sql .= 'LIKE ' . $dbh->quote("%${value}%");
		}else{
			$sql .= '= ' . $dbh->quote($value);
		}
		my $sth = $dbh->prepare($sql);
		$sth->execute;
		my @array = map { $_->[0] } @{$sth->fetchall_arrayref};
		alarm(0);
		return(\@array);
	};
	alarm(0);
	warn "$@\n" if($@);
}

sub name {
	my $me = shift;
	my $dbh = shift || die "name syntax error1";
	my $d_dbh = shift || die "name syntax error2";
	my $key = shift || die "name syntax error3";
	my $acct = shift || die "name syntax error4";

	if($acct eq '22' || $acct eq '2152938926'){
		return('Broadview Networks');
	}

	die "invalid key: $key\n" unless($key eq 'cust_id' || $key eq 'circuit_id');
	die "name invalid criteria: $acct\n" unless($acct =~ /^[a-z0-9]{2,40}$/i);

	my $sql = 'SELECT Last_name FROM oc8_cache_data';
	if($key eq 'cust_id'){
		$sql .= ' WHERE UidCustomerId = ' . $dbh->quote($acct);
	}else{
		$sql .= ',oc8_cache_ids WHERE oc8_cache_data.UidCustomerId = oc8_cache_ids.UidCustomerId';
		$sql .= ' AND oc8_cache_ids.Telephone_number = ' . $dbh->quote($acct);
	}
	my $sth = $d_dbh->prepare($sql);
	$sth->execute;
	my $row = $sth->fetchrow_arrayref;
	if($row->[0]){
		$row->[0] =~ s#/#_#g;
		return($row->[0]);
	}else{
		my $row = $me->all($dbh,$d_dbh,$key,$acct); # caches, and retrieves
		return($row->[1]);
	}
}

sub count {
	my $me = shift;
	my $dbh = shift;
	my $acct = shift || die "count syntax error2";

	eval {
		my $h = set_sig_handler( 'ALRM', sub { die "timeout!"; } );
		alarm(10);
		my $sql = 'SELECT COUNT(*) FROM profiles WITH(NOLOCK) WHERE UidCustomerId = ' . $dbh->quote($acct);
		my $sth = $dbh->prepare($sql);
		$sth->execute;
		my $row = $sth->fetchrow_arrayref;
		alarm(0);
		return($row->[0]);
	};
	alarm(0);
	warn "$@\n" if($@);
}

sub insert {
	die "will not insert into OC8 database\n";
	my $me = shift;
	my $dbh = shift;
	my $data = shift || die "insert syntax error";

	my (@keys,@values);
	while(my($key,$value) = each %{$data}){
		push @keys, $key;
		push @values, $value;
	}
	my $sql = 'INSERT INTO customers (';
	foreach my $key (@keys){
		$sql .= "$key,";
	}
	chop($sql);
	$sql .= ') VALUES (';
	foreach my $value (@values){
		$sql .= $dbh->quote($value) . ',';
	}
	chop($sql);
	$sql .= ')';
	
	my $sth = $dbh->prepare($sql);
	$sth->execute;
	return;
}

sub update {
	die "will not update OC8 database\n";
	my $me = shift;
	my $dbh = shift;
	my $acct = shift || die "update syntax erro1";
	my $data = shift || die "update syntax error2"; # hashref

	my $sql = 'UPDATE customers SET ';
	while(my($key,$value) = each %{$data}){
		$sql .= "$key = " . $dbh->quote($value) . ',';
	}
	chop($sql);
	$sql .= ' WHERE account_number = ' . $dbh->quote($acct);
	my $sth = $dbh->prepare($sql);
	$sth->execute;

	return;
}

sub status {
	my $me = shift;
	my $dbh = shift;
	my $d_dbh = shift;
	my $circuit_id = shift || die "status syntax error1";
	my $cust_id = shift || die "status syntax error2";

	die "invalid circuit_id: $circuit_id" unless($circuit_id =~ /^[a-z0-9]{2,40}$/i);
	die "invalid cust_id: $cust_id" unless($cust_id == int($cust_id));


	if($circuit_id eq '2152938926' || $cust_id == 22){
		return('Working');
	}

	my $sql = 'SELECT status FROM oc8_cache_ids WHERE Telephone_number = ' . $dbh->quote($circuit_id);
	$sql .= ' AND UidCustomerId = ' . $dbh->quote($cust_id);
	my $sth = $dbh->prepare($sql);
	my $row = $sth->fetchrow_arrayref;
	if($row->[0]){
		return($row->[0]);
	}else{
		my $row = $me->all($dbh,$d_dbh,'circuit_id',$circuit_id);
		return($row->[6]);
	}
}

sub all {
	my $me = shift;
	my $dbh = shift || die "all syntax error1";
	my $d_dbh = shift || die "all syntax error2";
	my $key = shift || die "all syntax error3";
	my $acct = shift || die "all syntax error4";

	die "all invalid criteria: $acct\n" unless($acct =~ /^[a-z0-9]{2,40}$/i);

	my %criteria = (cust_id => 'UidCustomerId', circuit_id => 'Telephone_number');
	die "invalid key: $key\n" unless($criteria{$key});

	# the order in $fields is important.  
	my $fields = 'UidCustomerId,Last_name,Mailing_street,Mailing_town,Mailing_state,Mailing_zip';
	$fields .= ',status' if($key eq 'circuit_id');

	my $sql = "SELECT oc8_cache_data.${fields} FROM oc8_cache_data";
	if($key eq 'cust_id'){
		$sql .= ' WHERE UidCustomerId = ' . $dbh->quote($acct);
	}else{
		$sql .= ',oc8_cache_ids WHERE oc8_cache_data.UidCustomerId = oc8_cache_ids.UidCustomerId';
		$sql .= ' AND oc8_cache_ids.Telephone_number = ' . $dbh->quote($acct);
	}
	my $sth = $d_dbh->prepare($sql);
	$sth->execute;
	my $row = $sth->fetchrow_arrayref;
	unless($row->[0]){
		eval {
			my $h = set_sig_handler( 'ALRM', sub { die "timeout!"; } );
			alarm(10);
			my $sql = "SELECT $fields FROM profiles WITH (NOLOCK) WHERE $criteria{$key} = " . $dbh->quote($acct);
			#$sql .= ' AND status != \'Closed\' AND status != \'Cancelled\''; # why did i do this? # jk 20071025
			# did it have something to do with me not understanding that status associatd to circuit id, not cust id??
			if(($key eq 'cust_id' && $acct == 22) || ($key eq 'circuit_id' && $acct eq '2152938926')){
				$sql .= ' AND Last_name = \'Broadview Network\''; #QQQ hack, IT SQL server is corrupt
			}
			warn "IT SQL [$sql]\n";
			my $sth = $dbh->prepare($sql);
			$sth->execute;
			$row = $sth->fetchrow_arrayref;
			alarm(0);
		};
		alarm(0);
		if($@){
			warn "$@\n";
			$row->[1] = "UNKNOWN";
		}elsif($row->[0]){
			$row->[1] =~ s#/#_#g;
	
			my $sqla = 'SELECT COUNT(*) FROM oc8_cache_data WHERE UidCustomerId = ' . $dbh->quote($row->[0]);
			my $stha = $d_dbh->prepare($sqla);
			$stha->execute;
			my $rowa = $stha->fetchrow_arrayref;
			unless($rowa->[0]){
				$fields =~ s/,status$//; # status goes in oc8_Cache_ids
				my $sql = "INSERT INTO oc8_cache_data ($fields,timestamp) VALUES (";
				$sql .= $dbh->quote($row->[0]) . ',';
				$sql .= $dbh->quote($row->[1]) . ',';
				$sql .= $dbh->quote($row->[2]) . ',';
				$sql .= $dbh->quote($row->[3]) . ',';
				$sql .= $dbh->quote($row->[4]) . ',';
				$sql .= $dbh->quote($row->[5]) . ',';
				$sql .= $dbh->quote(time()) . ')'; # timestamp
				my $sth = $d_dbh->prepare($sql);
				$sth->execute;
			}
			
			if($key eq 'circuit_id'){
				my $sql = 'SELECT status FROM oc8_cache_ids WHERE Telephone_number = ' . $dbh->quote($acct);
				$sql .= " AND UidCustomerId = $row->[0]";
				my $sth = $d_dbh->prepare($sql);
				$sth->execute;
				my $rowa = $sth->fetchrow_arrayref;
				if($rowa->[0]){
					$row->[6] = $rowa->[0];
				}else{
					my $rowb;
					eval {
						my $h = set_sig_handler( 'ALRM', sub { die "timeout!"; } );
						alarm(10);
						my $sqla = 'SELECT status FROM profiles WITH (NOLOCK) WHERE Telephone_number = ' . $dbh->quote($acct);
						$sqla .= " AND UidCustomerId = $row->[0]";
						my $stha = $dbh->prepare($sqla);
						$stha->execute;
						$rowb = $stha->fetchrow_arrayref;
						alarm(0);
					};
					alarm(0);
					if($@){
						warn "$@\n";
					}else{
						my $sql = 'INSERT INTO oc8_cache_ids (UidCustomerId,Telephone_number,status,timestamp) VALUES (';
						$sql .= $dbh->quote($row->[0]) . ',' . $dbh->quote($acct) . ',';
						$sql .= $dbh->quote($rowb->[0]) . ',' . $dbh->quote(time()) . ')';
						my $sth = $d_dbh->prepare($sql);
						$sth->execute;
 
						$row->[6] = $rowb->[0];
					}
				}
			}
		}else{
			warn "no result for $criteria{$key} = $acct\n";
			$row->[1] = "UNKNOWN";
		}
	}
	return($row);
}

return 1;
