#!/usr/local/bin/perl

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use Net::SMTP;
use IPAnalyze::IPmath;

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open config: $!\n";
while(<CONFIG>){
   next if(/^[#;]/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $DEBUG = 0;
my $VERBOSE = 0;

die "syntax error" unless($ARGV[0]);

my $ipmath = IPAnalyze::IPmath->new();

my $dbh;
until($dbh){
	$dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
	warn "$DBI::errstr\n" if($DBI::errstr);
	sleep 2;
}

foreach my $netid (@ARGV){
	my $stuff = 'netblock,cidr,usage_history,warn_level,tech_name,tech_email,enable_emailwar';
	my $sql = "SELECT $stuff FROM data WHERE netid = " . $dbh->quote($netid);
	my $sth = $dbh->prepare($sql);
	$sth->execute;
	while(my $row=$sth->fetchrow_arrayref){
		my $netblock = $row->[0];
		my $cidr = $row->[1];
		my $uh = $row->[2];
		my $wlvl = $row->[3];
		my $tech_name = $row->[4];
		my $email = $row->[5];
		my $enable = $row->[6];
	
		warn "[CHILD $$] Begining Probe of ${netblock}/${cidr} for ${netid}\n" if($DEBUG || $VERBOSE);
		my @result = split(/\n/, `/usr/local/sbin/fping -aq -r 1 -g ${netblock}/${cidr} 2>/dev/null`);
		my $r = @result;
	
		#Calculate Usage
		my $cu = 0;
		unless($r == 0){
			my $n = $ipmath->cidr_check($cidr);
			$cu = sprintf("%.0f", (($r * 100) / $n));
		}
	
		$uh .= "${cu}:"; # tack on current usage
	
		# only keep 30 days of history
		if($uh =~ /(([\d\.]+:){30})$/){
			$uh = $1;
		}
	
		my @history = split(/:/, $uh);
		my $count = @history;
		my $avu = 0;
		my $aa;
		if($count){
			$aa = 0;
			foreach(@history){
				if($_ > 100){
					warn "$netid has usage_history problem: $_\n";
					$count--; # we know at least one value is corrupt
				}else{
					$aa += $_;
				}
			}
			$avu = sprintf("%.0f",($aa/$count));
		}
	
		my $time = time();
		my $sqlu = 'UPDATE data SET current_usage = ' . $dbh->quote($cu);
		$sqlu .= ', average_usage = ' . $dbh->quote($avu);
		$sqlu .= ', usage_history = ' . $dbh->quote($uh);
		$sqlu .= ', last_probe = ' . $dbh->quote($time);
		$sqlu .= ', warn_level = ';
		if(($count == 30) && (($avu < 79) || (($cidr == 29) && ($avu < 63)))){
			$wlvl++;
			$sqlu .= $dbh->quote($wlvl);
		}else{
			$sqlu .= 0;
		}
		$sqlu .= ' WHERE netid = ' . $dbh->quote($netid);
		warn "[CHILD $$] would execute: $sqlu\n" if($DEBUG || $VERBOSE);
		unless($DEBUG){
			my $sthu = $dbh->prepare($sqlu);
			$sthu->execute;
		}
		#if(($wlvl) && ($enable eq 'YES')){
		if(0){ #QQQ
			warn "[CHILD $$] would send warning email: $wlvl\n" if($DEBUG || $VERBOSE);
			unless($DEBUG){
				if(open(RV, "$cfg{home_dir}/data/templates/text/warn_level${wlvl}")){
					my $smtp = Net::SMTP->new($cfg{mailhost});
					$smtp->mail($cfg{from});
					$smtp->recipient($email);
					$smtp->data();
					$smtp->datasend("To: \"${tech_name}\" <${email}>\n");
					$smtp->datasend("From: \"IPAnalyze\" <$cfg{from}>");
					$smtp->datasend("Subject: $cfg{'our_name'} IP Allocation ${netblock}/${cidr}");
					$smtp->datasend("X-NETID: ${netid}\n");
					chop(my $date=`date`);
					$smtp->datasend("Date: $date\n\n");
					while(<RV>) {
						$smtp->datasend($_);
					}
					close RV;
					$smtp->datasend("Customer Information:\n");
					$smtp->datasend("Network: ${netblock}/${cidr}\n");
					$smtp->datasend("Network usage on last probe:  $cu\%\n");
					$smtp->datasend("Average usage for the last 30 days:  ${avu}\%\n\n");
					$smtp->datasend("NOTE: This message has been sent from an automated account.\n");
					$smtp->datasend("If you have questions about this e-mail please call $cfg{'our_name'} at $cfg{'our_phone'}.\n");
					$smtp->datasend("You can also reach $cfg{'our_name'} by e-mail at $cfg{support_email}.\n");
					$smtp->dataend();
					$smtp->quit;
				}else{
					warn "could not open warn_level${wlvl}: $!\n";
				}
			}
		}
	}
}

$dbh->disconnect();
