#!/usr/local/bin/perl

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use CGI qw(:standard);
use IPAnalyze::Auth;
use IPAnalyze::Misc;
use IPAnalyze::Customers;
use Mail::VRFY;
use HTML::Template;

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open config: $!\n";
while(<CONFIG>){
	s/^(.*)\s*#.*/$1/;
	next if(/^$/);
	chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $d_dbh;
until($d_dbh){
	$d_dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
	warn "$DBI::errstr\n" if($DBI::errstr);
	sleep 1;
}
my $c_dbh = $d_dbh;
if($cfg{c_dsn} && $cfg{c_dbun} && $cfg{c_dbpw}){
	unless(($cfg{d_dsn} eq $cfg{c_dsn}) && ($cfg{d_dbun} eq $cfg{c_dbun}) && ($cfg{d_dbpw} eq $cfg{c_dbpw})){
		$c_dbh = undef;
		until($c_dbh){
			$c_dbh = DBI->connect ($cfg{c_dsn}, $cfg{c_dbun}, $cfg{c_dbpw}, {RaiseError => 1});
			warn "$DBI::errstr\n" if($DBI::errstr);
			sleep 1;
		}
	}
}

my $q = CGI->new();
my $auth = IPAnalyze::Auth->new();
my $misc = IPAnalyze::Misc->new();
my $customers = IPAnalyze::Customers->new();

my %areas = $misc->get_areas($d_dbh);

print $q->header(-expires=>'now');

if(my($username,$access) = $auth->check_access($d_dbh,$q,$cfg{app})){
	
	unless($access > 0){
		my @error = ("Access denied.");
		print $q->header(-expires=>'now');
		$misc->show_error($cfg{html_dir},\@error);
 		exit;
	}
	
	my $template = HTML::Template->new(filename => "$cfg{'html_dir'}/main.html");
	$template->param(OUR_COMPANY => $cfg{'our_name'});
	$template->param(REQUEST => 1);
	$template->param(SCRIPT => '/cgi-bin/ipanalyze.mpl');
  	$template->param(IPMANAGE => 1);
  	$template->param(IPMONITOR => 1);
  	$template->param(VERSION => $cfg{'version'});
  	$template->param(USERNAME => $username);

	if(my $action=$q->param('action')){
		if($action eq 'new'){
			print "cant make new customer via ipanalyze: build account in open cafe\n";
			exit;
			if($q->param('page_one')){
				my %cust = (company_name => $q->param('company_name'),
				            cust_id => $q->param('cust_id'),
				            address => $q->param('address'),
				            city => $q->param('city'),
				            state => $q->param('state'),
				            zip => $q->param('zip_code'),
				            country => $q->param('country'),
				);

				my @error;
				unless($cust{company_name} =~ /^[A-Z0-9\s\-\.]+$/i) {
				      push @error , "Invalid Characters used for Company Name";
				}
				if($cust{company_name} =~ /^\s/ || $cust{company_name} =~ /\s$/){
				   push @error, "Company Names dont start or end with a space";
				}
				if($cust{address} !~ /^[A-Z0-9\s\-\,\.]+$/i) {
				      push @error , "Invalid Characters used for Address";
				}
				if($cust{city} !~ /^[A-Z0-9\s\-\.]+$/i) {
				      push @error , "Invalid Characters used for city";
				}
				unless($cust{state} =~ /^[a-z]+$/i){
				      push @error , "Invalid Characters used for state";
				}
				unless($cust{zip} =~ /^\d{5}$/){
				      push @error , "Invalid Zip Code: must be 5 numbers";
				}
				if($cust{country} !~ /^[A-Z]{2}$/ ) {
				      push @error , "Invalid Country code: must be 2 letter ISO 3166-1 code ";
				}
				unless($cust{account_number} =~ /^\d{10}$/i) {
				      push @error , "Invalid Account Number";
				}

				my $count = $customers->count($c_dbh,$cust{'account_number'});
				if($count){
					push @error, "Account Number already used";
				}

				if (@error) {
				   my $template = HTML::Template->new(filename => "$cfg{'html_dir'}/error.html");
				   my @loop_data = ();
				   while (@error) {
						push @loop_data, {
							ERROR_DATA => shift @error,
						};
				   }
				   $template->param(ERROR_LOOP => \@loop_data);
				   print $template->output;
				   exit;
				}

				$customers->insert($c_dbh,\%cust);

				$template->param(CUSTOMER_ENTERED => 1);
				$template->param(CUST_ID => $cust{'cust_id'});

				
			}else{
				$template->param(GATHER_CUST_INFO => 1);
			}
		}else{
			die "confuzzled\n";
		}
	}elsif(my $circuit_id=$q->param('circuit_id')){
		my ($cust,@error);
		$circuit_id =~ s/^\s*//g;
		$circuit_id =~ s/\s*$//g;
		if($circuit_id =~ /^\d{6}[A-Z0-9]{4}$/i){
			$cust = $customers->all($c_dbh,$d_dbh,'circuit_id',$circuit_id);
		}else{
			push @error , "Invalid Characters used for Circuit ID";
		}
		if((my $cidr=$q->param('cidr')) && (my $le=$q->param('local_email')) &&
		   (my $lc=$q->param('local_contact')) && (my $area = $q->param('area')) &&
		   (my $network_use=$q->param('network_use'))){

			my($tn,$te,$tp,$tx);
			if($network_use eq '1'){
				if($q->param('tech_phone') || $q->param('tech_email') || $q->param('tech_name') || $q->param('tech_ext')){
					if($q->param('tech_phone') && $q->param('tech_email') && $q->param('tech_name')){
						$tn = $q->param('tech_name');
						$te = $q->param('tech_email');
						$tp = $q->param('tech_phone');
						$tx = $q->param('tech_ext');
					}else{
						push @error, "Need ALL tech contact info or NO tech contact info for WAN assignments.";
					}
				}else{
					# we can/will provide tech contact on WANs
					$tn = $cfg{'our_name'};
					$te = $cfg{'support_email'};
					$tp = $cfg{'our_phone'};
				}
			}else{
				$tn = $q->param('tech_name');
				$te = $q->param('tech_email');
				$tp = $q->param('tech_phone');
				$tx = $q->param('tech_ext');
			}

			unless($lc =~ /\S+\s+\S+/){
				push @error, "\"Your Name\" - not \"alias\" or \"nickname\" or \"username\".";
			}
			unless($network_use eq '1' || $network_use eq '2'){
				push @error, "Bad Network Use.";
			}
			if(($network_use eq '1') && ($cidr ne '30' && $cidr ne '32')){
				push @error, "WAN assignments must be /30 or /32";
			}
			if($le eq $te){
				push @error, "Provide a valid Technical Contact.";
			}
			unless($lc =~ /^[A-Z0-9\s\-\.\']+$/i ){
				push @error, "Invalid Characters used for Local Contact Name";
			}
			my $local_chk = Mail::VRFY::CheckAddress($le);
			if($local_chk){
			      push @error, "Invalid Local Contact E-mail (code $local_chk): Please check address.";
			}
			unless($tn =~ /^[A-Z0-9\s\.-]+$/i) {
			      push @error , "Invalid Characters used for Technical Contact Name";
			}
		
			if(($access >= 6) && ($te =~ /^:/)){
				# skip the tech email tests
				substr($te,0,1) = '';
			}else{
				#if($te =~ /\@broadviewnet.com$/i){
				#	push @error, "Technical Contact is clearly not via broadviewnet.com";
				#}else{
					#unless(($network_use eq '1' && $te eq $cfg{'support_email'}) ||
					#       ($te eq 'abuse@broadviewnet.net') ||
					#       ($te eq 'isp-noc@broadviewnet.net')){
					#	# no more than one ocurrance of this address per custid
					#	my $sql = 'SELECT cust_id FROM data WHERE tech_email = ' . $d_dbh->quote($te);
					#	my $sth = $d_dbh->prepare($sql);
					#	$sth->execute;
					#	my %hash;
					#	while(my $row = $sth->fetchrow_arrayref){
					#		$hash{$row->[0]} ++;
					#	}
					#	my $count = keys %hash;
					#	if($count == 1){
					#		unless(exists($hash{$cust->[0]})){
					#			my ($id) = keys %hash;
					#			push @error, "Technical Contact already used by the following customer_ids: $id";
					#		}
					#	}elsif($count > 1){
					#		my $string = 'Technical Contact already used by the following customer_ids:';
					#		foreach my $key (keys %hash){
					#			$string .= "${key},";
					#		}	
					#		chop($string);
					#		push @error, $string;
					#	}
					#}
					
				#}
				unless($network_use eq '1' && $te eq $cfg{'support_email'}){
					if($te =~ /broadviewnet\./ && $te ne 'ipadmin-proxy@broadviewnet.net'){
						push @error, "tech contact may not be via broadviewnet\.(net|com)";
					}elsif(($te =~ /\@kister.net/ || $te =~ /\@jeremykister.com/) && ($username != 'jkister')){
						push @error, "configuration error"; # idiots
					}else{
						my ($domain) = $te =~ /\@(\S+\.\S+)$/; # poor man's regex QQQ
						my $ok = 1;
						if(open(BADMAIL, "$cfg{home_dir}/data/badmail.txt")){
							while(<BADMAIL>){
								if(/^${domain}$/i){
									$ok=0;
									last;
								}
							}
							close BADMAIL;
						}
						if($ok){
							my $tech_chk = Mail::VRFY::CheckAddress(addr => $te, timeout => 50);
							if($tech_chk){
							      push @error, "Invalid Technical Contact E-mail (code $tech_chk): Please check email address.";
							}
						}else{
							push @error, "Cannot use $te as technical contact email: domain rejected.";
						}
					}
				}
			}
			
			$tp =~ s/-//g;
			unless($tp =~ /^\+?\d{10,15}$/) {
			      push @error, "Invalid Characters used for Technical Phone Number: use format 2155551212";
			}
			unless($tx eq ''){
				unless(($tx =~ /^\d{1,10}$/) && ($tx <= 2147483647)){
					push @error, "Invalid extension";
				}
			}
			unless($cidr =~ /^\d{1,2}$/){
				push @error, "Invalid CIDR";
			}
			unless($cidr <= 32 && $cidr >= 24){
				push @error, "Insane CIDR";
			}
			unless($area == int($area)){
				push @error, "Invalid Area";
			}
			unless(exists($areas{id}{$area})){
				push @error, "Invalid Area ID";
			}
			if (@error) {
			   my $template = HTML::Template->new(filename => "$cfg{'html_dir'}/error.html");
			   my @loop_data = ();
			   while (@error) {
			      my %row_data;
			      $row_data{ERROR_DATA} = shift @error;
			      push(@loop_data, \%row_data);
			   }
			   $template->param(ERROR_LOOP => \@loop_data);
			   print $template->output;
			   exit;
			}

			my $time = time();
			my $sql = 'INSERT INTO data (cust_id,circuit_id,area,cidr,network_use,local_contact,local_email,';
			$sql .= 'tech_name,tech_email,tech_phone,tech_ext,status,bornondate) VALUES (';
			$sql .= $d_dbh->quote($cust->[0]) . ',' . $d_dbh->quote($circuit_id) . ',' . $d_dbh->quote($area) . ',';
			$sql .= $d_dbh->quote($cidr) . ',' . $d_dbh->quote($network_use) . ',' . $d_dbh->quote($lc) . ',';
			$sql .= $d_dbh->quote($le) . ',' . $d_dbh->quote($tn) . ',' . $d_dbh->quote($te) . ',';
			$sql .= $d_dbh->quote($tp);
			if(defined($tx)){
				$sql .= ',' . $d_dbh->quote($tx);
			}else{
				$sql .= ',NULL';
			}
			if($cidr eq '30' || $cidr eq '29'){
				$sql .= ',2,';
			}else{
				$sql .= ',1,';
			}
			$sql .= $d_dbh->quote($time) . ')';
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;

			$sql = 'SELECT netid FROM data WHERE circuit_id = ' . $d_dbh->quote($circuit_id);
			$sql .= ' AND area = ' . $d_dbh->quote($area);
			$sql .= ' AND bornondate = ' . $d_dbh->quote($time);
			$sql .= ' AND cust_id = ' . $d_dbh->quote($cust->[0]);
			$sql .= ' AND cidr = ' . $d_dbh->quote($cidr);
			$sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $row = $sth->fetchrow_arrayref;
			my $netid = $row->[0];

			$template->param(REQUEST_ENTERED => 1);
			$template->param(NETID => $netid);

			if($cidr eq '30' || $cidr eq '29'){
				my $mail_template = "$cfg{home_dir}/data/templates/ipanalyze/text/request_approve.txt";
				$misc->mail_update($d_dbh,$c_dbh,$customers,$netid,$mail_template,\%cfg);
				sleep 6; # just a bit of help for slow mail servers that are about to bounce the email QQQ
			}
		}else{
			$template->param(CIRCUIT_ID => $circuit_id);
			if(defined($cust->[0])){
				my $country = $cust->[7] || 'US';
		 		$template->param(CUST_ID => $cust->[0]);
				$template->param(CUST_NAME => $cust->[1]);
				$template->param(ADDRESS => $cust->[2]);
				$template->param(CITY => $cust->[3]);
				$template->param(STATE => $cust->[4]);
				$template->param(ZIP => $cust->[5]);
				$template->param(STATUS => $cust->[6]);
				$template->param(COUNTRY => $country);
				my @areas;
				while(my($key,$value) = each %{$areas{id}}){
					my %hash;
					$hash{AREA_ID} = $key;
					$hash{AREA_NAME} = $value;
					push @areas, \%hash;
				}
				$template->param(AREAS => \@areas);
			}else{
				$template->param(NOTFOUND => 1);
				$template->param(CUSTOMER_INQUIRE => 1);
			}
		}
	}else{
		$template->param(CUSTOMER_INQUIRE => 1);
	}
	print $template->output;
}else{
	print $q->start_html(),
			"bad sess_id\n",
			$q->end_html();
}
